EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApiKeys_CreateOrUpdateApiKey',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ApiKey_Id',
    /* Description_Text        */ 'Creates or updates the specified API key.'
GO

IF OBJECT_ID('[ApiKeys_CreateOrUpdateApiKey]') IS NOT NULL DROP PROCEDURE [ApiKeys_CreateOrUpdateApiKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApiKeys_CreateOrUpdateApiKey]
(
    @ApiKey_Text VARCHAR(50),
    @ApiKey_Description NVARCHAR(MAX),
    @ApiKey_Configuration XML,
    @ApiKey_Name NVARCHAR(100),
    @ApiKeyTasks_Table [dbo].[ApiKeyTaskEntry] READONLY,
    @ApiKey_Id INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

    IF NOT EXISTS(SELECT * FROM [ApiKeys] WITH (UPDLOCK) WHERE [ApiKey_Id] = @ApiKey_Id)
    BEGIN
        
        INSERT INTO [ApiKeys] 
        (
            [ApiKey_Text]
           ,[ApiKey_Description] 
           ,[ApiKey_Configuration]
           ,[ApiKey_Name]
        )
        VALUES 
        (
            @ApiKey_Text
           ,NULLIF(@ApiKey_Description, '')
           ,@ApiKey_Configuration
           ,@ApiKey_Name
        )

        SET @ApiKey_Id = SCOPE_IDENTITY()

    END
    ELSE
    BEGIN

	    UPDATE [ApiKeys]
           SET [ApiKey_Text] = @ApiKey_Text,
               [ApiKey_Description] = NULLIF(@ApiKey_Description, ''),
               [ApiKey_Configuration] = @ApiKey_Configuration,
               [ApiKey_Name] = @ApiKey_Name
         WHERE [ApiKey_Id] = @ApiKey_Id

     END

	-- use CTE as filter to prevent full table scan
	;WITH Filtered_ApiKeyTasks
	AS
	(
		SELECT *
		  FROM [ApiKeyTasks]
		 WHERE [ApiKey_Id] = @ApiKey_Id
	)
	MERGE Filtered_ApiKeyTasks AS TARGET
	USING @ApiKeyTasks_Table AS SOURCE
	   ON (
		       (TARGET.[Feed_Id] = SOURCE.[Feed_Id] OR (TARGET.[Feed_Id] IS NULL AND SOURCE.[Feed_Id] IS NULL))
		   AND (TARGET.[FeedGroup_Id] = SOURCE.[FeedGroup_Id] OR (TARGET.[FeedGroup_Id] iS NULL AND SOURCE.[FeedGroup_Id] IS NULL))
           AND (TARGET.[Task_Id] = SOURCE.[Task_Id])
		  )

	 WHEN NOT MATCHED BY TARGET THEN
		INSERT
		(
            [ApiKey_Id],
            [Feed_Id],
            [FeedGroup_Id],
            [Task_Id]
		)
		VALUES
		(
			@ApiKey_Id,
			SOURCE.[Feed_Id],
			SOURCE.[FeedGroup_Id],
			SOURCE.[Task_Id]
		)

	 WHEN NOT MATCHED BY SOURCE THEN
		DELETE
	;

END
GO

GRANT EXECUTE ON [ApiKeys_CreateOrUpdateApiKey] TO [ProGetUser_Role]