EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Assets_FindFromPath',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'AssetItem_Id,CanonicalAssetPath_Text,Inherited_CacheConfiguration_XML',
    /* Description_Text        */ 'Finds an asset with the specified full path and returns its id, properly-cased path, and cache configuration.',
	/* Remarks_Text            */ 'Path must use forward slash (/) and not have any leading or trailing slashes. Paths are case-sensitive unless AllowCaseInsensitiveMatch_Indicator is Y.'
GO

IF OBJECT_ID('[Assets_FindFromPath]') IS NOT NULL DROP PROCEDURE [Assets_FindFromPath]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Assets_FindFromPath]
(
	@Feed_Id INT,
	@AssetPath_Text NVARCHAR(MAX),
	@IncludeDeleted_Indicator YNINDICATOR,
	@AllowCaseInsensitiveMatch_Indicator YNINDICATOR,
	
	@AssetItem_Id INT = NULL OUT,
	@CanonicalAssetPath_Text NVARCHAR(MAX) = NULL OUT,
	@Inherited_CacheConfiguration_XML XML = NULL OUT
)
AS
BEGIN

	DECLARE @EndPos INT = 1
	WHILE (@EndPos <= LEN(@AssetPath_Text)) BEGIN
	    
		-- start +1 after the last '/', or at the first char
	    DECLARE @StartPos INT = COALESCE(LEN(@CanonicalAssetPath_Text + '/') + 1 , 1) 
		
		-- end at the next '/', or after the path string
		SET @EndPos = CHARINDEX('/', @AssetPath_Text + '/', @StartPos) 

		-- pull between the slashes
		DECLARE @Name NVARCHAR(200) = SUBSTRING(@AssetPath_Text, @StartPos, @EndPos - @StartPos)

		SELECT @AssetItem_Id = [AssetItem_Id],
		       @CanonicalAssetPath_Text = COALESCE(@CanonicalAssetPath_Text + '/', '') + [AssetItem_Name],
			   @Inherited_CacheConfiguration_XML = COALESCE([CacheConfiguration_XML], @Inherited_CacheConfiguration_XML)
          FROM [AssetItems]
  		 WHERE [Feed_Id] = @Feed_Id
		   AND (@AssetItem_Id IS NULL OR [Parent_AssetItem_Id] = @AssetItem_Id)
		   AND [AssetItem_Lower_Name] = LOWER(@Name)
		   AND (@AllowCaseInsensitiveMatch_Indicator = 'Y' OR [AssetItem_Name] = @Name)
		   AND (@IncludeDeleted_Indicator = 'Y' OR [Deleted_Date] IS NULL)

		IF @@ROWCOUNT = 0 BEGIN
			SET @AssetItem_Id = NULL
			SET @CanonicalAssetPath_Text = NULL
			SET @Inherited_CacheConfiguration_XML = NULL
			BREAK
		END
	END


END
GO

GRANT EXECUTE ON [Assets_FindFromPath] TO [ProGetUser_Role]