EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExternalUsers_CreateOrUpdateUser',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates or modifies an external user.'
GO

IF OBJECT_ID('[ExternalUsers_CreateOrUpdateUser]') IS NOT NULL DROP PROCEDURE [ExternalUsers_CreateOrUpdateUser]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [ExternalUsers_CreateOrUpdateUser]
(
	@UserDirectory_Id INT,
	@ExternalUser_Name NVARCHAR(200),
	@ExternalUser_Display_Name NVARCHAR(250),
	@ExternalUser_Email_Address NVARCHAR(250)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [ExternalGroups]
				WITH (UPDLOCK)
			   WHERE [UserDirectory_Id] = @UserDirectory_Id
			     AND [ExternalGroup_Name] = @ExternalUser_Name)
		RAISERROR('Cannot create external user "%s"; there is a group of the same name.', 16, 1, @ExternalUser_Name)

	IF EXISTS(SELECT *
				FROM [ExternalUsers]
				WITH (UPDLOCK)
			   WHERE [UserDirectory_Id] = @UserDirectory_Id
			     AND [ExternalUser_Name] = @ExternalUser_Name)
	BEGIN

		UPDATE [ExternalUsers]
		   SET [ExternalUser_Display_Name] = @ExternalUser_Display_Name,
			   [ExternalUser_Email_Address] = @ExternalUser_Email_Address
		 WHERE [UserDirectory_Id] = @UserDirectory_Id
		   AND [ExternalUser_Name] = @ExternalUser_Name

	END
	ELSE
	BEGIN

		INSERT INTO [ExternalUsers]
		(
			[UserDirectory_Id],
			[ExternalUser_Name],
			[ExternalUser_Display_Name],
			[ExternalUser_Email_Address]
		)
		VALUES
		(
			@UserDirectory_Id,
			@ExternalUser_Name,
			@ExternalUser_Display_Name,
			@ExternalUser_Email_Address
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ExternalUsers_CreateOrUpdateUser] TO [ProGetUser_Role]
GO