EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'MavenArtifacts_SetFileProperties',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Modifies Maven file metadata.'
GO

IF OBJECT_ID('[MavenArtifacts_SetFileProperties]') IS NOT NULL DROP PROCEDURE [MavenArtifacts_SetFileProperties]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [MavenArtifacts_SetFileProperties]
(
	@Feed_Id INT,
	@GroupId_Text NVARCHAR(200),
	@ArtifactId_Text NVARCHAR(100),
	@Version_Text NVARCHAR(50),
	@File_Type NVARCHAR(50),
	@FileName_Text NVARCHAR(100),
	@Updated_Date DATETIME = NULL,
	@Download_Count INT = NULL,
	@LastDownloaded_Date DATETIME = NULL
)
AS
BEGIN

SET NOCOUNT ON
SET QUOTED_IDENTIFIER ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE MAF
	   SET MAF.[Updated_Date] = COALESCE(@Updated_Date, MAF.[Updated_Date]),
	       MAF.[Download_Count] = COALESCE(@Download_Count, MAF.[Download_Count]),
	       MAF.[LastDownloaded_Date] = CASE WHEN @Download_Count = 0 THEN NULL ELSE COALESCE(@LastDownloaded_Date, MAF.[LastDownloaded_Date]) END
	  FROM [MavenArtifactFiles] MAF
	       INNER JOIN [MavenArtifacts] MA
		           ON MA.[MavenArtifact_Id] = MAF.[MavenArtifact_Id]
	 WHERE MA.[Feed_Id] = @Feed_Id
	   AND MA.[GroupId_Text] = @GroupId_Text
	   AND MA.[ArtifactId_Text] = @ArtifactId_Text
	   AND MAF.[Version_Text] = @Version_Text
	   AND MAF.[File_Type] = @File_Type
	   AND COALESCE(MAF.[FileName_Text], @ArtifactId_Text + '-' + @Version_Text) = @FileName_Text

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [MavenArtifacts_SetFileProperties] TO [ProGetUser_Role]