EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Packages_GetPackages',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Packages_Combined',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of combined packages filtering by the specified criteria.'
GO

IF OBJECT_ID('[Packages_GetPackages]') IS NOT NULL DROP PROCEDURE [Packages_GetPackages]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Packages_GetPackages]
(
	@FeedIds_Csv VARCHAR(MAX) = NULL,
	@PackgeIdSearch_Text VARCHAR(MAX) = NULL,

	@Cached_Indicator YNINDICATOR = NULL,
	@OrderBy_Name  VARCHAR(MAX) = NULL, /* Package, Date, Downloads */
	@OrderByDesc_Indicator YNINDICATOR = NULL,
	@Skip_Count INT = NULL,
	@Take_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SET @Skip_Count = COALESCE(@Skip_Count, 0)
	SET @Take_Count = COALESCE(@Take_Count, 50)

	/* This is a bit of a disaster due to SQL limitations
	 * [1] using sp_executesql would require us to grant SELECT permissions on the appropriate tables
	 * [2] ROW_NUMBER() OVER's ORDER BY doesn't like constant or CASE'd expressions  (conversion errors)
	 *
	 * The best bet is probably to go with sp_Executesql, but that breaks our model. So, we will just do 
	 * 6 copy/paste statements that are slightly tweaked. Dynamic SQL Version below
	 */

	IF @OrderBy_Name = 'Package' AND @OrderByDesc_Indicator = 'N'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY COALESCE([Group_Id],'') + [Package_Id])
			  FROM [Packages_Combined]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@PackgeIdSearch_Text IS NULL OR [Package_Id] LIKE '%' + @PackgeIdSearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count

	ELSE IF @OrderBy_Name = 'Package' AND @OrderByDesc_Indicator = 'Y'

		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY COALESCE([Group_Id],'') + [Package_Id] DESC)
			  FROM [Packages_Combined]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count
			   AND (@PackgeIdSearch_Text IS NULL OR [Package_Id] LIKE '%' + @PackgeIdSearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)

	ELSE IF @OrderBy_Name = 'Downloads' AND @OrderByDesc_Indicator = 'N'

		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Download_Count])
			  FROM [Packages_Combined]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@PackgeIdSearch_Text IS NULL OR [Package_Id] LIKE '%' + @PackgeIdSearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count

	ELSE IF @OrderBy_Name = 'Downloads' AND @OrderByDesc_Indicator = 'Y'
		
		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Download_Count] DESC)
			  FROM [Packages_Combined]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@PackgeIdSearch_Text IS NULL OR [Package_Id] LIKE '%' + @PackgeIdSearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count
		 
	ELSE IF @OrderBy_Name = 'Date' AND @OrderByDesc_Indicator = 'N'

		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Published_Date])
			  FROM [Packages_Combined]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@PackgeIdSearch_Text IS NULL OR [Package_Id] LIKE '%' + @PackgeIdSearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count

	ELSE

		WITH FilteredPackages AS  
		(  
			SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY [Published_Date] DESC)
			  FROM [Packages_Combined]
			 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT) FROM dbo.CsvToTable(@FeedIds_Csv, ','))
			   AND (@PackgeIdSearch_Text IS NULL OR [Package_Id] LIKE '%' + @PackgeIdSearch_Text + '%')
			   AND (@Cached_Indicator IS NULL OR [Cached_Indicator] = @Cached_Indicator)
		)
		SELECT * FROM FilteredPackages
		 WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count


	--DECLARE @orderbysql NVARCHAR(MAX) = 
	--   CASE @OrderBy_Name 
	--   WHEN 'Feed' THEN '[Feed_Id]'
	--   WHEN 'Package' THEN 'COALESCE([Group_Id],'''') + [Package_Id]'
	--   WHEN 'Downloads' THEN '[Download_Count]'
	--   ELSE 'Published_Date' END
	--   +
	--   CASE @OrderByDesc_Indicator WHEN 'Y' THEN ' DESC' ELSE ' ASC' END
	--   +
	--   ', [Published_Date] DESC'

	--DECLARE @statement NVARCHAR(MAX) =  N'
	--EXECUTE AS CALLER
	--;WITH FilteredPackages AS  
	--(  
	--	SELECT *, [Row_Number] = ROW_NUMBER() OVER (ORDER BY ' + @orderbysql + N') 
	--	  FROM [Packages_Combined]
	--	 WHERE [Feed_Id] IN (SELECT CAST([Value_Text] AS INT)
	--						   FROM dbo.CsvToTable(@FeedIds_Csv, '',''))
	--)
	--SELECT * FROM FilteredPackages
	-- WHERE [Row_Number] BETWEEN @Skip_Count AND @Skip_Count + @Take_Count'
	-- EXEC sp_executesql @statement,
	--				   N'@Skip_Count INT, @Take_Count INT, @FeedIds_Csv VARCHAR(MAX)',
	--				   @Skip_Count, @Take_Count, @FeedIds_Csv

	--SET @statement = @ctesql + N'SELECT @TotalPackage_Count = COUNT(*) FROM FilteredPackages '
	--EXEC sp_executesql @statement, 
	--				   N'@FeedIds_Csv VARCHAR(MAX), @TotalPackage_Count INT OUTPUT',
	--				   @FeedIds_Csv, @TotalPackage_Count OUTPUT
END
GO

GRANT EXECUTE ON [Packages_GetPackages] TO [ProGetUser_Role]
GO